#include <iostream>
#include <complex>
#include <fstream>

using namespace std;

const double PI = 3.14159265358979;

double LED(double _w){
	const double sigma = 12.5;	//強度をsqrtするので
	return 1/sqrt(2*PI)/sigma * exp( -0.5*pow(_w/sigma,2.0) );
}

int main(){

	const int N = 1000;	//関数のデータ点
	const int N_s = 100;	//単スリットの分割数
	const int N_w = 100;	//ダブルスリットの分割数
	const int N_l = 100;

	const double x_max = 4.0;
	const double x_min = .0;

	const double c = 0.8;
	const double a = 0.5;
	const double b = 0.15;
	const double d = 5.0;

	const double L_1 = 200.0;
	const double L_2 = 500.0;
	const double L_0 = 70.0;

	const double lambda = 0.000637;
	const double k = 2*PI/lambda;

	ofstream ofs("conv_doubleslit.txt");

	double norm;

	for(int i=0;i<N;i++){
		//complex<double> f(.0,.0);
		double f2 = .0;
		double x = x_min + (x_max-x_min)*i/N;
		for(int m=0;m<N_l;m++){
			double dw = d/N_l;
			double w = -d/2 + m*dw;

			complex<double> g(.0,.0);

			for(int l=0;l<N_w;l++){	//ダブルスリット
				double dv = b/N_w;
				double v = a/2 - b/2 + l*dv;

				complex<double> t1(.0, k*v*(v-x)/L_2);
				complex<double> t2(.0, k*(-v)*(-v-x)/L_2);

				for(int j=0;j<N_s;j++){	//単スリット
					double du = c/N_s;
					double u = -c/2 + j*du;

					complex<double> s1(.0, k*u*( (u-v)/L_1 + (u-w)/L_0 ));
					complex<double> s2(.0, k*u*( (u+v)/L_1 + (u-w)/L_0 ));

					g += LED(w)*dw * (exp(s1+t1) + exp(s2+t2));
				}
			}
			f2 += pow(abs(g),2.0);
		}
		//f2 /= (N_w*N_s*2.0);
		if(i==0){norm = f2;}
		f2/=norm;

		cout << "x = " << x << ", |f(x)|^2 = " << f2 << endl;
		ofs << x << " " << f2 << endl;
	}


	return 0;
}
